#include "bsp.h"

/* private variables */
static uint32_t GPIO_PORT[LEDn] = {LED1_GPIO_PORT, LED1_GPIO_PORT, LED2_GPIO_PORT, LED3_GPIO_PORT,
                                   LED4_GPIO_PORT, LED5_GPIO_PORT, LED6_GPIO_PORT, LED7_GPIO_PORT};
static uint32_t GPIO_PIN[LEDn] = {LED0_PIN, LED1_PIN, LED2_PIN, LED3_PIN, LED4_PIN, LED5_PIN, LED6_PIN, LED7_PIN};

static rcu_periph_enum COM_CLK[COMn] = {EVAL_COM0_CLK};
static uint32_t COM_TX_PIN[COMn] = {EVAL_COM0_TX_PIN};
static uint32_t COM_RX_PIN[COMn] = {EVAL_COM0_RX_PIN};

static rcu_periph_enum GPIO_CLK[LEDn] = {LED0_GPIO_CLK, LED1_GPIO_CLK, LED2_GPIO_CLK, LED3_GPIO_CLK,
                                         LED4_GPIO_CLK, LED5_GPIO_CLK, LED6_GPIO_CLK, LED7_GPIO_CLK};

static uint32_t KEY_PORT[KEYn] = {SET_KEY_GPIO_PORT, 
                                  UP_KEY_GPIO_PORT,
                                  DOWN_KEY_GPIO_PORT,
																	RIGHT_KEY_GPIO_PORT,
																	LEFT_KEY_GPIO_PORT};

static uint32_t KEY_PIN[KEYn] = {SET_KEY_PIN, UP_KEY_PIN, DOWN_KEY_PIN, RIGHT_KEY_PIN, LEFT_KEY_PIN};
static rcu_periph_enum KEY_CLK[KEYn] = {SET_KEY_GPIO_CLK, 
                                        UP_KEY_GPIO_CLK,
                                        DOWN_KEY_GPIO_CLK,
                                        RIGHT_KEY_GPIO_CLK,
                                        LEFT_KEY_GPIO_CLK};
static exti_line_enum KEY_EXTI_LINE[KEYn] = {SET_KEY_EXTI_LINE,
                                             UP_KEY_EXTI_LINE,
                                             DOWN_KEY_EXTI_LINE,
                                             RIGHT_KEY_EXTI_LINE,
                                             LEFT_KEY_EXTI_LINE};
static uint8_t KEY_PORT_SOURCE[KEYn] = {SET_KEY_EXTI_PORT_SOURCE,
                                        UP_KEY_EXTI_PORT_SOURCE,
                                        DOWN_KEY_EXTI_PORT_SOURCE,
                                        RIGHT_KEY_EXTI_PORT_SOURCE,
                                        LEFT_KEY_EXTI_PORT_SOURCE};
static uint8_t KEY_PIN_SOURCE[KEYn] = {SET_KEY_EXTI_PIN_SOURCE,
                                       UP_KEY_EXTI_PIN_SOURCE,
                                       DOWN_KEY_EXTI_PIN_SOURCE,
                                       RIGHT_KEY_EXTI_PIN_SOURCE,
                                       LEFT_KEY_EXTI_PIN_SOURCE};
static uint8_t KEY_IRQn[KEYn] = {SET_KEY_EXTI_IRQn, 
                                 UP_KEY_EXTI_IRQn,
                                 DOWN_KEY_EXTI_IRQn,
                                 RIGHT_KEY_EXTI_IRQn,
                                 LEFT_KEY_EXTI_IRQn}; 

/*!
    \brief    configure led GPIO
    \param[in]  lednum: specify the Led to be configured
			\arg        LED0
			\arg        LED1
			\arg        LED2
      \arg        LED3
      \arg        LED4
      \arg        LED5
			\arg        LED6
			\arg        LED7
    \param[out] none
    \retval     none
*/
void  gd_eval_led_init (led_typedef_enum lednum)
{
    /* enable the led clock */
    rcu_periph_clock_enable(GPIO_CLK[lednum]);
    /* configure led GPIO port */ 
    gpio_mode_set(GPIO_PORT[lednum], GPIO_MODE_OUTPUT, GPIO_PUPD_NONE,GPIO_PIN[lednum]);
    gpio_output_options_set(GPIO_PORT[lednum], GPIO_OTYPE_PP, GPIO_OSPEED_50MHZ,GPIO_PIN[lednum]);

    GPIO_BC(GPIO_PORT[lednum]) = GPIO_PIN[lednum];
}

/*!
    \brief    turn on selected led
    \param[in]  lednum: specify the Led to be turned on
			\arg        LED0
			\arg        LED1
			\arg        LED2
      \arg        LED3
      \arg        LED4
      \arg        LED5
			\arg        LED6
			\arg        LED7
    \param[out] none
    \retval     none
*/
void gd_eval_led_on(led_typedef_enum lednum)
{
    GPIO_BOP(GPIO_PORT[lednum]) = GPIO_PIN[lednum];
}

/*!
    \brief    turn off selected led
    \param[in]  lednum: specify the Led to be turned off
			\arg        LED0
			\arg        LED1
			\arg        LED2
      \arg        LED3
      \arg        LED4
      \arg        LED5
			\arg        LED6
			\arg        LED7
    \param[out] none
    \retval     none
*/
void gd_eval_led_off(led_typedef_enum lednum)
{
    GPIO_BC(GPIO_PORT[lednum]) = GPIO_PIN[lednum];
}

/*!
    \brief    toggle selected led
    \param[in]  lednum: specify the Led to be toggled
			\arg        LED0
			\arg        LED1
			\arg        LED2
      \arg        LED3
      \arg        LED4
      \arg        LED5
			\arg        LED6
			\arg        LED7
    \param[out] none
    \retval     none
*/
void gd_eval_led_toggle(led_typedef_enum lednum)
{
    GPIO_TG(GPIO_PORT[lednum]) = GPIO_PIN[lednum];
}

/*!
    \brief    configure key
    \param[in]  key_num: specify the key to be configured
      \arg        KEY_SET: set key
      \arg        KEY_UP: up key
      \arg        KEY_DOWN: down key
      \arg        KEY_RIGHT: right key
      \arg        KEY_LEFT: left key
    \param[in]  key_mode: specify button mode
      \arg        KEY_MODE_GPIO: key will be used as simple IO
      \arg        KEY_MODE_EXTI: key will be connected to EXTI line with interrupt
    \param[out] none
    \retval     none
*/
void gd_eval_key_init(key_typedef_enum key_num, keymode_typedef_enum key_mode)
{
    /* enable the key clock */
    rcu_periph_clock_enable(KEY_CLK[key_num]);
    rcu_periph_clock_enable(RCU_SYSCFG);

    /* configure button pin as input */
    gpio_mode_set(KEY_PORT[key_num], GPIO_MODE_INPUT, GPIO_PUPD_NONE,KEY_PIN[key_num]);

    if (key_mode == KEY_MODE_EXTI) {
        /* enable and set key EXTI interrupt to the lowest priority */
        nvic_irq_enable(KEY_IRQn[key_num], 2U, 0U);

        /* connect key EXTI line to key GPIO pin */
        syscfg_exti_line_config(KEY_PORT_SOURCE[key_num], KEY_PIN_SOURCE[key_num]);

        /* configure key EXTI line */
        exti_init(KEY_EXTI_LINE[key_num], EXTI_INTERRUPT, EXTI_TRIG_FALLING);
        exti_interrupt_flag_clear(KEY_EXTI_LINE[key_num]);
    }
}

/*!
    \brief    return the selected button state
    \param[in]  button: specify the button to be checked
      \arg        KEY_SET: set key
      \arg        KEY_UP: up key
      \arg        KEY_DOWN: down key
      \arg        KEY_RIGHT: right key
      \arg        KEY_LEFT: left key
    \param[out] none
    \retval     the button GPIO pin value
*/
uint8_t gd_eval_key_state_get(key_typedef_enum button)
{
    return gpio_input_bit_get(KEY_PORT[button], KEY_PIN[button]);
}

/*!
    \brief    configure COM port
    \param[in]  COM: COM on the board
      \arg        EVAL_COM0: COM on the board
    \param[out] none
    \retval     none
*/
void gd_eval_com_init(uint32_t com)
{
    /* enable GPIO clock */
    uint32_t COM_ID = 0;
    if(EVAL_COM0 == com)
    {
        COM_ID = 0U;
    }

    rcu_periph_clock_enable( EVAL_COM0_GPIO_CLK);

    /* enable USART clock */
    rcu_periph_clock_enable(COM_CLK[COM_ID]);

    /* connect port to USARTx_Tx */
    gpio_af_set(EVAL_COM0_GPIO_PORT, EVAL_COM0_AF, COM_TX_PIN[COM_ID]);

    /* connect port to USARTx_Rx */
    gpio_af_set(EVAL_COM0_GPIO_PORT, EVAL_COM0_AF, COM_RX_PIN[COM_ID]);

    /* configure USART Tx as alternate function push-pull */
    gpio_mode_set(EVAL_COM0_GPIO_PORT, GPIO_MODE_AF, GPIO_PUPD_PULLUP,COM_TX_PIN[COM_ID]);
    gpio_output_options_set(EVAL_COM0_GPIO_PORT, GPIO_OTYPE_PP, GPIO_OSPEED_50MHZ,COM_TX_PIN[COM_ID]);

    /* configure USART Rx as alternate function push-pull */
    gpio_mode_set(EVAL_COM0_GPIO_PORT, GPIO_MODE_AF, GPIO_PUPD_PULLUP,COM_RX_PIN[COM_ID]);
    gpio_output_options_set(EVAL_COM0_GPIO_PORT, GPIO_OTYPE_PP, GPIO_OSPEED_50MHZ,COM_RX_PIN[COM_ID]);

    /* USART configure */
    usart_deinit(com);
    usart_baudrate_set(com,115200U);
    usart_receive_config(com, USART_RECEIVE_ENABLE);
    usart_transmit_config(com, USART_TRANSMIT_ENABLE);
    usart_enable(com);
}

