#ifndef BSP_H
#define BSP_H

#ifdef __cplusplus
 extern "C" {
#endif

#include "gd32f4xx.h"
     
/* exported types */
typedef enum 
{
    LED0 = 0,
    LED1 = 1,
	  LED2 = 2,
    LED3 = 3,
	  LED4 = 4,
    LED5 = 5,
	  LED6 = 6,
    LED7 = 7
} led_typedef_enum;

typedef enum 
{
    KEY_SET   = 0,
    KEY_UP    = 1,
    KEY_DOWN  = 2,
	  KEY_RIGHT = 3,
	  KEY_LEFT  = 4
} key_typedef_enum;

typedef enum 
{
    KEY_MODE_GPIO = 0,
    KEY_MODE_EXTI = 1
} keymode_typedef_enum;

/* eval board low layer led */
#define LEDn                             8U

#define LED0_PIN                         GPIO_PIN_3
#define LED0_GPIO_PORT                   GPIOE
#define LED0_GPIO_CLK                    RCU_GPIOE

#define LED1_PIN                         GPIO_PIN_4
#define LED1_GPIO_PORT                   GPIOE
#define LED1_GPIO_CLK                    RCU_GPIOE
  
#define LED2_PIN                         GPIO_PIN_13
#define LED2_GPIO_PORT                   GPIOC
#define LED2_GPIO_CLK                    RCU_GPIOC
  
#define LED3_PIN                         GPIO_PIN_0
#define LED3_GPIO_PORT                   GPIOF
#define LED3_GPIO_CLK                    RCU_GPIOF

#define LED4_PIN                         GPIO_PIN_1
#define LED4_GPIO_PORT                   GPIOF
#define LED4_GPIO_CLK                    RCU_GPIOF

#define LED5_PIN                         GPIO_PIN_2
#define LED5_GPIO_PORT                   GPIOF
#define LED5_GPIO_CLK                    RCU_GPIOF

#define LED6_PIN                         GPIO_PIN_3
#define LED6_GPIO_PORT                   GPIOF
#define LED6_GPIO_CLK                    RCU_GPIOF

#define LED7_PIN                         GPIO_PIN_0
#define LED7_GPIO_PORT                   GPIOC
#define LED7_GPIO_CLK                    RCU_GPIOC

#define COMn                             1U
#define EVAL_COM0                        USART2
#define EVAL_COM0_CLK                    RCU_USART2

#define EVAL_COM0_TX_PIN                 GPIO_PIN_10
#define EVAL_COM0_RX_PIN                 GPIO_PIN_11

#define EVAL_COM0_GPIO_PORT              GPIOC
#define EVAL_COM0_GPIO_CLK               RCU_GPIOC
#define EVAL_COM0_AF                     GPIO_AF_7

#define KEYn                             5U

/* SET push-button */
#define SET_KEY_PIN                   GPIO_PIN_15
#define SET_KEY_GPIO_PORT             GPIOG
#define SET_KEY_GPIO_CLK              RCU_GPIOG
#define SET_KEY_EXTI_LINE             EXTI_15
#define SET_KEY_EXTI_PORT_SOURCE      EXTI_SOURCE_GPIOG
#define SET_KEY_EXTI_PIN_SOURCE       EXTI_SOURCE_PIN15
#define SET_KEY_EXTI_IRQn             EXTI10_15_IRQn

/* UP push-button */
#define UP_KEY_PIN                   GPIO_PIN_5
#define UP_KEY_GPIO_PORT             GPIOB
#define UP_KEY_GPIO_CLK              RCU_GPIOB
#define UP_KEY_EXTI_LINE             EXTI_5
#define UP_KEY_EXTI_PORT_SOURCE      EXTI_SOURCE_GPIOB
#define UP_KEY_EXTI_PIN_SOURCE       EXTI_SOURCE_PIN5
#define UP_KEY_EXTI_IRQn             EXTI0_IRQn  

/* DOWN push-button */
#define DOWN_KEY_PIN                     GPIO_PIN_6
#define DOWN_KEY_GPIO_PORT               GPIOB
#define DOWN_KEY_GPIO_CLK                RCU_GPIOB
#define DOWN_KEY_EXTI_LINE               EXTI_6
#define DOWN_KEY_EXTI_PORT_SOURCE        EXTI_SOURCE_GPIOB
#define DOWN_KEY_EXTI_PIN_SOURCE         EXTI_SOURCE_PIN6
#define DOWN_KEY_EXTI_IRQn               EXTI0_IRQn

/* RIGHT push-button */
#define RIGHT_KEY_PIN                     GPIO_PIN_7
#define RIGHT_KEY_GPIO_PORT               GPIOB
#define RIGHT_KEY_GPIO_CLK                RCU_GPIOB
#define RIGHT_KEY_EXTI_LINE               EXTI_7
#define RIGHT_KEY_EXTI_PORT_SOURCE        EXTI_SOURCE_GPIOB
#define RIGHT_KEY_EXTI_PIN_SOURCE         EXTI_SOURCE_PIN7
#define RIGHT_KEY_EXTI_IRQn               EXTI0_IRQn

/* LEFT push-button */
#define LEFT_KEY_PIN                     GPIO_PIN_1
#define LEFT_KEY_GPIO_PORT               GPIOE
#define LEFT_KEY_GPIO_CLK                RCU_GPIOE
#define LEFT_KEY_EXTI_LINE               EXTI_1
#define LEFT_KEY_EXTI_PORT_SOURCE        EXTI_SOURCE_GPIOE
#define LEFT_KEY_EXTI_PIN_SOURCE         EXTI_SOURCE_PIN1
#define LEFT_KEY_EXTI_IRQn               EXTI0_IRQn

/* function declarations */
/* configures led GPIO */
void gd_eval_led_init(led_typedef_enum lednum);
/* turn on selected led */
void gd_eval_led_on(led_typedef_enum lednum);
/* turn off selected led */
void gd_eval_led_off(led_typedef_enum lednum);
/* toggle the selected led */
void gd_eval_led_toggle(led_typedef_enum lednum);
/* configure key */
void gd_eval_key_init(key_typedef_enum key_num, keymode_typedef_enum key_mode);
/* return the selected button state */
uint8_t gd_eval_key_state_get(key_typedef_enum button);
/* configure COM port */
void gd_eval_com_init(uint32_t com);


#ifdef __cplusplus
}
#endif

#endif /* BSP_H */
